@@ -39,6 +39,8 @@ class TokenType(AutoName):
     COLON = auto()
     DOTCOLON = auto()
     DCOLON = auto()
+    DCOLONDOLLAR = auto()
+    DCOLONPERCENT = auto()
     DQMARK = auto()
     SEMICOLON = auto()
     STAR = auto()
@@ -53,6 +55,8 @@ class TokenType(AutoName):
     NEQ = auto()
     NULLSAFE_EQ = auto()
     COLON_EQ = auto()
+    COLON_GT = auto()
+    NCOLON_GT = auto()
     AND = auto()
     OR = auto()
     AMP = auto()
@@ -76,10 +80,14 @@ class TokenType(AutoName):
     AT_GT = auto()
     DOLLAR = auto()
     PARAMETER = auto()
+    SESSION = auto()
     SESSION_PARAMETER = auto()
     DAMP = auto()
     XOR = auto()
     DSTAR = auto()
+    QMARK_AMP = auto()
+    QMARK_PIPE = auto()
+    HASH_DASH = auto()
 
     URI_START = auto()
 
@@ -183,6 +191,7 @@ class TokenType(AutoName):
     DATEMULTIRANGE = auto()
     UUID = auto()
     GEOGRAPHY = auto()
+    GEOGRAPHYPOINT = auto()
     NULLABLE = auto()
     GEOMETRY = auto()
     POINT = auto()
@@ -228,7 +237,6 @@ class TokenType(AutoName):
     # keywords
     ALIAS = auto()
     ALTER = auto()
-    ALWAYS = auto()
     ALL = auto()
     ANTI = auto()
     ANY = auto()
@@ -301,7 +309,6 @@ class TokenType(AutoName):
     HINT = auto()
     IGNORE = auto()
     ILIKE = auto()
-    ILIKE_ANY = auto()
     IN = auto()
     INDEX = auto()
     INNER = auto()
@@ -322,7 +329,6 @@ class TokenType(AutoName):
     LATERAL = auto()
     LEFT = auto()
     LIKE = auto()
-    LIKE_ANY = auto()
     LIMIT = auto()
     LIST = auto()
     LOAD = auto()
@@ -373,6 +379,7 @@ class TokenType(AutoName):
     RENAME = auto()
     REPLACE = auto()
     RETURNING = auto()
+    REVOKE = auto()
     REFERENCES = auto()
     RIGHT = auto()
     RLIKE = auto()
@@ -419,6 +426,9 @@ class TokenType(AutoName):
     WINDOW = auto()
     WITH = auto()
     UNIQUE = auto()
+    UTC_DATE = auto()
+    UTC_TIME = auto()
+    UTC_TIMESTAMP = auto()
     VERSION_SNAPSHOT = auto()
     TIMESTAMP_SNAPSHOT = auto()
     OPTION = auto()
@@ -705,7 +715,6 @@ class Tokenizer(metaclass=_Tokenizer):
         "~~*": TokenType.ILIKE,
         "~*": TokenType.IRLIKE,
         "ALL": TokenType.ALL,
-        "ALWAYS": TokenType.ALWAYS,
         "AND": TokenType.AND,
         "ANTI": TokenType.ANTI,
         "ANY": TokenType.ANY,
@@ -831,6 +840,7 @@ class Tokenizer(metaclass=_Tokenizer):
         "SCHEMA": TokenType.SCHEMA,
         "SELECT": TokenType.SELECT,
         "SEMI": TokenType.SEMI,
+        "SESSION": TokenType.SESSION,
         "SET": TokenType.SET,
         "SETTINGS": TokenType.SETTINGS,
         "SHOW": TokenType.SHOW,
@@ -969,6 +979,7 @@ class Tokenizer(metaclass=_Tokenizer):
         "COMMENT": TokenType.COMMENT,
         "EXPLAIN": TokenType.COMMAND,
         "GRANT": TokenType.GRANT,
+        "REVOKE": TokenType.REVOKE,
         "OPTIMIZE": TokenType.COMMAND,
         "PREPARE": TokenType.COMMAND,
         "VACUUM": TokenType.COMMAND,
@@ -1422,7 +1433,7 @@ class Tokenizer(metaclass=_Tokenizer):
         self._advance(len(start))
         text = self._extract_string(end, raw_string=token_type == TokenType.RAW_STRING)
 
-        if base:
+        if base and text:
             try:
                 int(text, base)
             except Exception:
