@@ -8,6 +8,10 @@ class TestPostgres(Validator):
     dialect = "postgres"
 
     def test_postgres(self):
+        self.validate_identity(
+            "select count() OVER(partition by a order by a range offset preceding exclude current row)",
+            "SELECT COUNT() OVER (PARTITION BY a ORDER BY a range BETWEEN offset preceding AND CURRENT ROW EXCLUDE CURRENT ROW)",
+        )
         expr = self.parse_one("SELECT * FROM r CROSS JOIN LATERAL UNNEST(ARRAY[1]) AS s(location)")
         unnest = expr.args["joins"][0].this.this
         unnest.assert_is(exp.Unnest)
@@ -805,6 +809,10 @@ FROM json_data, field_ids""",
         )
         self.assertIsInstance(self.parse_one("id::UUID"), exp.Cast)
 
+        self.validate_identity('1::"int"', "CAST(1 AS INT)").to.is_type(exp.DataType.Type.INT)
+        self.validate_identity(
+            '1::"udt"', 'CAST(1 AS "udt")'
+        ).to.this == exp.DataType.Type.USERDEFINED
         self.validate_identity(
             "COPY tbl (col1, col2) FROM 'file' WITH (FORMAT format, HEADER MATCH, FREEZE TRUE)"
         )
@@ -931,6 +939,24 @@ FROM json_data, field_ids""",
         self.validate_identity("SELECT * FROM foo WHERE id = %(id_param)s")
         self.validate_identity("SELECT * FROM foo WHERE id = ?")
 
+        self.validate_identity("a ?| b").assert_is(exp.JSONBContainsAnyTopKeys)
+        self.validate_identity(
+            """SELECT '{"a":1, "b":2, "c":3}'::jsonb ?| array['b', 'c']""",
+            """SELECT CAST('{"a":1, "b":2, "c":3}' AS JSONB) ?| ARRAY['b', 'c']""",
+        )
+
+        self.validate_identity("a ?& b").assert_is(exp.JSONBContainsAllTopKeys)
+        self.validate_identity(
+            """SELECT '["a", "b"]'::jsonb ?& array['a', 'b']""",
+            """SELECT CAST('["a", "b"]' AS JSONB) ?& ARRAY['a', 'b']""",
+        )
+
+        self.validate_identity("a #- b").assert_is(exp.JSONBDeleteAtPath)
+        self.validate_identity(
+            """SELECT '["a", {"b":1}]'::jsonb #- '{1,b}'""",
+            """SELECT CAST('["a", {"b":1}]' AS JSONB) #- '{1,b}'""",
+        )
+
     def test_ddl(self):
         # Checks that user-defined types are parsed into DataType instead of Identifier
         self.parse_one("CREATE TABLE t (a udt)").this.expressions[0].args["kind"].assert_is(
@@ -1498,3 +1524,101 @@ CROSS JOIN JSON_ARRAY_ELEMENTS(CAST(JSON_EXTRACT_PATH(tbox, 'boxes') AS JSON)) A
         for key_type in ("FOR SHARE", "FOR UPDATE", "FOR NO KEY UPDATE", "FOR KEY SHARE"):
             with self.subTest(f"Test lock type {key_type}"):
                 self.validate_identity(f"SELECT 1 FROM foo AS x {key_type} OF x")
+
+    def test_grant(self):
+        grant_cmds = [
+            "GRANT SELECT ON TABLE users TO role1",
+            "GRANT INSERT, DELETE ON TABLE orders TO user1",
+            "GRANT SELECT ON employees TO manager WITH GRANT OPTION",
+            "GRANT USAGE ON SCHEMA finance TO user2",
+            "GRANT ALL PRIVILEGES ON DATABASE mydb TO PUBLIC",
+            "GRANT CREATE ON SCHEMA public TO developer",
+            "GRANT CONNECT ON DATABASE testdb TO readonly_user",
+            "GRANT TEMPORARY ON DATABASE testdb TO temp_user",
+            "GRANT TRIGGER ON orders TO audit_role",
+            "GRANT REFERENCES ON products TO foreign_key_user",
+            "GRANT TRUNCATE ON logs TO admin_role",
+            "GRANT UPDATE(salary) ON employees TO hr_manager",
+            "GRANT SELECT(id, name), UPDATE(email) ON customers TO customer_service",
+        ]
+
+        for sql in grant_cmds:
+            with self.subTest(f"Testing PostgreSQL's GRANT statement: {sql}"):
+                self.validate_identity(sql)
+
+        self.validate_identity(
+            "GRANT EXECUTE ON FUNCTION calculate_bonus(integer) TO analyst",
+            "GRANT EXECUTE ON FUNCTION CALCULATE_BONUS(integer) TO analyst",
+        )
+
+        advanced_grants = [
+            "GRANT INSERT, DELETE ON ALL TABLES IN SCHEMA myschema TO user1",
+            "GRANT developer_role TO john",
+            "GRANT admin_role TO mary WITH ADMIN OPTION",
+        ]
+
+        for sql in advanced_grants:
+            with self.subTest(f"Testing PostgreSQL's advanced GRANT statement: {sql}"):
+                self.validate_identity(sql, check_command_warning=True)
+
+    def test_revoke(self):
+        revoke_cmds = [
+            "REVOKE SELECT ON TABLE users FROM role1",
+            "REVOKE INSERT, DELETE ON TABLE orders FROM user1",
+            "REVOKE USAGE ON SCHEMA finance FROM user2",
+            "REVOKE ALL PRIVILEGES ON DATABASE mydb FROM PUBLIC",
+            "REVOKE CREATE ON SCHEMA public FROM developer",
+            "REVOKE CONNECT ON DATABASE testdb FROM readonly_user",
+            "REVOKE TEMPORARY ON DATABASE testdb FROM temp_user",
+            "REVOKE TRIGGER ON orders FROM audit_role",
+            "REVOKE REFERENCES ON products FROM foreign_key_user",
+            "REVOKE TRUNCATE ON logs FROM admin_role",
+            "REVOKE USAGE ON SCHEMA finance FROM user2 CASCADE",
+            "REVOKE SELECT ON TABLE orders FROM user1 RESTRICT",
+            "REVOKE GRANT OPTION FOR SELECT ON employees FROM manager",
+            "REVOKE GRANT OPTION FOR SELECT ON employees FROM manager RESTRICT",
+            "REVOKE UPDATE(salary) ON employees FROM hr_manager",
+            "REVOKE SELECT(id, name), UPDATE(email) ON customers FROM customer_service",
+        ]
+
+        for sql in revoke_cmds:
+            with self.subTest(f"Testing PostgreSQL's REVOKE statement: {sql}"):
+                self.validate_identity(sql)
+
+        self.validate_identity(
+            "REVOKE EXECUTE ON FUNCTION calculate_bonus(integer) FROM analyst",
+            "REVOKE EXECUTE ON FUNCTION CALCULATE_BONUS(integer) FROM analyst",
+        )
+
+        advanced_revoke_cmds = [
+            "REVOKE INSERT, DELETE ON ALL TABLES IN SCHEMA myschema FROM user1",
+            "REVOKE developer_role FROM john",
+            "REVOKE admin_role FROM mary",
+        ]
+
+        for sql in advanced_revoke_cmds:
+            with self.subTest(f"Testing PostgreSQL's advanced REVOKE statement: {sql}"):
+                self.validate_identity(sql, check_command_warning=True)
+
+    def test_begin_transaction(self):
+        self.validate_all(
+            "BEGIN",
+            write={
+                "postgres": "BEGIN",
+                "presto": "START TRANSACTION",
+                "trino": "START TRANSACTION",
+            },
+        )
+
+        for keyword in ("TRANSACTION", "WORK"):
+            for level in (
+                "ISOLATION LEVEL SERIALIZABLE",
+                "ISOLATION LEVEL READ COMMITTED",
+                "NOT DEFFERABLE",
+                "READ WRITE",
+                "DEFERRABLE",
+            ):
+                with self.subTest(f"Testing Postgres's BEGIN {keyword} {level}"):
+                    self.validate_identity(
+                        f"BEGIN {keyword} {level}, {level}", f"BEGIN {level}, {level}"
+                    ).assert_is(exp.Transaction)
