@@ -511,7 +511,7 @@ class TestPresto(Validator):
                 "duckdb": "CREATE TABLE test AS SELECT 1",
                 "presto": "CREATE TABLE test WITH (format='PARQUET') AS SELECT 1",
                 "hive": "CREATE TABLE test STORED AS PARQUET AS SELECT 1",
-                "spark": "CREATE TABLE test USING PARQUET AS SELECT 1",
+                "spark": "CREATE TABLE test STORED AS PARQUET AS SELECT 1",
             },
         )
         self.validate_all(
@@ -794,7 +794,6 @@ class TestPresto(Validator):
                 "databricks": "ANY_VALUE(x)",
                 "doris": "ANY_VALUE(x)",
                 "drill": "ANY_VALUE(x)",
-                "duckdb": "ANY_VALUE(x)",
                 "hive": "FIRST(x)",
                 "mysql": "ANY_VALUE(x)",
                 "oracle": "ANY_VALUE(x)",
@@ -1335,3 +1334,29 @@ MATCH_RECOGNIZE (
     def test_analyze(self):
         self.validate_identity("ANALYZE tbl")
         self.validate_identity("ANALYZE tbl WITH (prop1=val1, prop2=val2)")
+
+    def test_bit_aggs(self):
+        self.validate_all(
+            "BITWISE_AND_AGG(x)",
+            read={
+                "presto": "BITWISE_AND_AGG(x)",
+                "trino": "BITWISE_AND_AGG(x)",
+                "oracle": "BITWISE_AND_AGG(x)",
+            },
+        )
+        self.validate_all(
+            "BITWISE_OR_AGG(x)",
+            read={
+                "presto": "BITWISE_OR_AGG(x)",
+                "trino": "BITWISE_OR_AGG(x)",
+                "oracle": "BITWISE_OR_AGG(x)",
+            },
+        )
+        self.validate_all(
+            "BITWISE_XOR_AGG(x)",
+            read={
+                "presto": "BITWISE_XOR_AGG(x)",
+                "trino": "BITWISE_XOR_AGG(x)",
+                "oracle": "BITWISE_XOR_AGG(x)",
+            },
+        )
