@@ -284,6 +284,7 @@ SELECT * FROM test FETCH FIRST ROWS ONLY
 SELECT * FROM test FETCH FIRST 1 ROWS ONLY
 SELECT * FROM test ORDER BY id DESC FETCH FIRST 10 ROWS WITH TIES
 SELECT * FROM test ORDER BY id DESC FETCH FIRST 10 PERCENT ROWS WITH TIES
+SELECT * FROM test ORDER BY always DESC
 SELECT * FROM test FETCH NEXT 1 ROWS ONLY
 SELECT (1 > 2) AS x FROM test
 SELECT NOT (1 > 2) FROM test
@@ -552,6 +553,8 @@ CREATE TABLE a.b AS SELECT a FROM a.c
 CREATE TABLE IF NOT EXISTS x AS SELECT a FROM d
 CREATE TABLE z (a INT, b VARCHAR, c VARCHAR(100), d DECIMAL(5, 3))
 CREATE TABLE z (end INT)
+CREATE TABLE z (bucket INT)
+CREATE TABLE z (truncate INT)
 CREATE TABLE z (a ARRAY<TEXT>, b MAP<TEXT, DOUBLE>, c DECIMAL(5, 3))
 CREATE TABLE z (a INT, b VARCHAR COMMENT 'z', c VARCHAR(100) COMMENT 'z', d DECIMAL(5, 3))
 CREATE TABLE z (a INT(11) DEFAULT UUID())
@@ -883,6 +886,12 @@ GRANT SELECT, INSERT ON FUNCTION tbl TO user
 GRANT SELECT ON orders TO ROLE PUBLIC
 GRANT SELECT ON nation TO alice WITH GRANT OPTION
 GRANT DELETE ON SCHEMA finance TO bob
+REVOKE SELECT ON TABLE tbl FROM user
+REVOKE SELECT, INSERT ON FUNCTION tbl FROM user
+REVOKE SELECT ON orders FROM ROLE PUBLIC
+REVOKE GRANT OPTION FOR SELECT ON nation FROM alice
+REVOKE DELETE ON SCHEMA finance FROM bob CASCADE
+REVOKE INSERT ON TABLE orders FROM user RESTRICT
 SELECT attach
 SELECT detach
 SELECT 1 OFFSET 1
@@ -895,3 +904,30 @@ SELECT ARG_MAX(DISTINCT selected_col, filtered_col) FROM table
 SELECT ARG_MIN(DISTINCT selected_col, filtered_col) FROM table
 a.b.c.D()
 a.b.c.d.e.f.G()
+SELECT CUME_DIST() OVER (ORDER BY 1) FROM (SELECT 1)
+SELECT DENSE_RANK() OVER (ORDER BY 1) FROM (SELECT 1)
+SELECT NTILE(1) OVER (ORDER BY 1) FROM (SELECT 1)
+SELECT RANK() OVER (ORDER BY 1) FROM (SELECT 1)
+SELECT PERCENT_RANK() OVER (ORDER BY 1) FROM (SELECT 1)
+SELECT ACOS(x)
+SELECT ACOSH(x)
+SELECT ASIN(x)
+SELECT ASINH(x)
+SELECT ATAN(x)
+SELECT ATANH(x)
+SELECT ATAN2(x, y)
+SELECT COT(x)
+SELECT COTH(x)
+SELECT CSC(x)
+SELECT CSCH(x)
+SELECT SEC(x)
+SELECT SECH(x)
+SELECT UTC_DATE
+SELECT UTC_TIME
+SELECT UTC_TIMESTAMP
+SELECT SIN(x)
+SELECT SINH(x)
+SELECT COSINE_DISTANCE(v1, v2)
+SELECT EUCLIDEAN_DISTANCE(v1, v2)
+FOO(values.c)
+case.*
